import threading

# pylint:disable=abstract-method
from dvc_objects.fs.base import FileSystem
from funcy import cached_property, wrap_prop


class WebHDFSFileSystem(FileSystem):
    protocol = "webhdfs"
    REQUIRES = {"fsspec": "fsspec"}
    PARAM_CHECKSUM = "checksum"

    @classmethod
    def _strip_protocol(cls, path: str) -> str:
        from fsspec.utils import infer_storage_options

        return infer_storage_options(path)["path"]

    def unstrip_protocol(self, path: str) -> str:
        host = self.fs_args["host"]
        port = self.fs_args["port"]
        path = path.lstrip("/")
        return f"webhdfs://{host}:{port}/{path}"

    @staticmethod
    def _get_kwargs_from_urls(urlpath):
        from fsspec.implementations.webhdfs import WebHDFS

        return (
            WebHDFS._get_kwargs_from_urls(  # pylint:disable=protected-access
                urlpath
            )
        )

    def _prepare_credentials(self, **config):
        self._ssl_verify = config.pop("ssl_verify", True)
        principal = config.pop("kerberos_principal", None)
        if principal:
            config["kerb_kwargs"] = {"principal": principal}
        return config

    @wrap_prop(threading.Lock())
    @cached_property
    def fs(self):
        from fsspec.implementations.webhdfs import WebHDFS

        fs = WebHDFS(**self.fs_args)
        fs.session.verify = self._ssl_verify
        return fs

    def checksum(self, path):
        ukey = self.fs.ukey(path)
        return ukey["bytes"]



from dvc.testing.fixtures import *  # noqa, pylint: disable=wildcard-import,unused-import
from dvc_hdfs.tests.fixtures import (  # noqa, pylint: disable=unused-import
    docker_compose_file,
    hadoop,
    hdfs_server,
)

from .fixtures import *  # noqa, pylint: disable=wildcard-import,unused-import



from contextlib import contextmanager

from dvc.testing.cloud import Cloud
from dvc.testing.path_info import CloudURLInfo


class WebHDFS(Cloud, CloudURLInfo):  # pylint: disable=abstract-method
    @contextmanager
    def _webhdfs(self):
        from hdfs import InsecureClient

        client = InsecureClient(
            f"http://{self.host}:{self.port}", self.user, root="/"
        )
        yield client

    def is_file(self):
        with self._webhdfs() as _hdfs:
            return _hdfs.status(self.path)["type"] == "FILE"

    def is_dir(self):
        with self._webhdfs() as _hdfs:
            return _hdfs.status(self.path)["type"] == "DIRECTORY"

    def exists(self):
        with self._webhdfs() as _hdfs:
            return _hdfs.status(self.path, strict=False) is not None

    def mkdir(self, mode=0o777, parents=False, exist_ok=False):
        assert mode == 0o777
        assert parents

        with self._webhdfs() as _hdfs:
            # NOTE: hdfs.makekdirs always creates parents
            _hdfs.makedirs(self.path, permission=mode)

    def write_bytes(self, contents):
        with self._webhdfs() as _hdfs:
            with _hdfs.write(self.path, overwrite=True) as writer:
                writer.write(contents)

    def read_bytes(self):
        with self._webhdfs() as _hdfs:
            with _hdfs.read(self.path) as reader:
                return reader.read()

    @property
    def config(self):
        return {"url": self.url}

    @property
    def fs_path(self):
        return "/" + self.path.lstrip("/")



import uuid

import pytest

from .cloud import WebHDFS


@pytest.fixture
def make_webhdfs(hdfs_server):
    def _make_webhdfs():
        port = hdfs_server["webhdfs"]
        url = f"webhdfs://127.0.0.1:{port}/{uuid.uuid4()}"
        return WebHDFS(url)

    return _make_webhdfs


@pytest.fixture
def webhdfs(make_webhdfs):
    return make_webhdfs()



import pytest
from dvc.testing.test_api import TestAPI  # noqa, pylint: disable=unused-import
from dvc.testing.test_remote import (  # noqa, pylint: disable=unused-import
    TestRemote,
)
from dvc.testing.test_workspace import TestAdd as _TestAdd
from dvc.testing.test_workspace import TestImport as _TestImport


@pytest.fixture
def remote(make_remote):
    yield make_remote(name="upstream", typ="webhdfs")


@pytest.fixture
def workspace(make_workspace):
    yield make_workspace(name="workspace", typ="webhdfs")


class TestImport(_TestImport):
    @pytest.fixture
    def stage_md5(self):
        return "3869631c193f0a3c206c6f04e84cb2b6"

    @pytest.fixture
    def is_object_storage(self):
        return False

    @pytest.fixture
    def dir_md5(self):
        return "32f6597da5c3c1dead9dc562faac09a2.dir"


class TestAdd(_TestAdd):
    @pytest.fixture
    def hash_name(self):
        return "checksum"

    @pytest.fixture
    def hash_value(self):
        return (
            "000002000000000000000000a86fe4d846edc1bf4c355cb6112f141e00000000"
        )

    @pytest.fixture
    def dir_hash_value(self):
        pytest.skip("external outputs are broken for hdfs dirs")
